/*
 * Copyright (c) 2003-2004 OFFIS. This program is made available under the terms of
 * the MIT License.
 */
package org.easymock.classextension;

import java.lang.reflect.Method;

import org.easymock.MockControl;
import org.easymock.classextension.internal.ClassProxyFactory;
import org.easymock.internal.IBehaviorFactory;

/**
 * Instances of <code>MockClassControl</code> control the behavior of their
 * associated mock objects. For more information, see the EasyMock
 * documentation.
 * 
 * @see <a href="http://www.easymock.org/">EasyMock </a>
 */
public class MockClassControl extends MockControl {

    /**
     * Creates a mock control object for the specified class or interface. The
     * {@link MockClassControl}and its associated mock object will not check
     * the order of expected method calls. An unexpected method call on the mock
     * object will lead to an <code>AssertionFailedError</code>.
     * 
     * @param classToMock
     *            the class to mock.
     * @return the mock control (which is a {@link MockClassControl}instance)
     */
    public static MockControl createControl(Class classToMock) {
        if (classToMock.isInterface()) {
            return MockControl.createControl(classToMock);
        }
        return new MockClassControl(classToMock, UNORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * Same as {@link #createControl(Class)}but allows to pass a list of
     * methods to mock. All the other methods won't be. It means that if these
     * methods are called, their real code will be executed.
     * 
     * @param classToMock
     *            the class to mock
     * @param mockedMethods
     *            Methods to be mocked. If null, all methods will be mocked.
     * @return the mock control
     */
    public static MockClassControl createControl(Class classToMock,
            Method[] mockedMethods) {
        return new MockClassControl(classToMock, mockedMethods,
                UNORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * Creates a mock control object for the specified class or interface. The
     * {@link MockClassControl}and its associated mock object will check the
     * order of expected method calls. An unexpected method call on the mock
     * object will lead to an <code>AssertionFailedError</code>.
     * 
     * @param classToMock
     *            the class to mock.
     * @return the mock control (which is a {@link MockClassControl}instance)
     */
    public static MockControl createStrictControl(Class classToMock) {
        if (classToMock.isInterface()) {
            return MockControl.createStrictControl(classToMock);
        }
        return new MockClassControl(classToMock, ORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * Same as {@link #createStrictControl(Class)}but allows to pass a list of
     * methods to mock. All the other methods won't be. It means that if these
     * methods are called, their real code will be executed.
     * 
     * @param classToMock
     *            the class to mock
     * @param mockedMethods
     *            Methods to be mocked. If null, all methods will be mocked.
     * @return the mock control
     */
    public static MockClassControl createStrictControl(Class classToMock,
            Method[] mockedMethods) {
        return new MockClassControl(classToMock, mockedMethods,
                ORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * Creates a mock control object for the specified class or interface. The
     * {@link MockClassControl}and its associated mock object will check not
     * the order of expected method calls. An unexpected method call on the mock
     * object will return an empty value (0, null, false).
     * 
     * @param classToMock
     *            the class to mock.
     * @return the mock control (which is a {@link MockClassControl}instance)
     */
    public static MockControl createNiceControl(Class classToMock) {
        if (classToMock.isInterface()) {
            return MockControl.createNiceControl(classToMock);
        }
        return new MockClassControl(classToMock, NICE_BEHAVIOR_FACTORY);
    }

    /**
     * Same as {@link #createNiceControl(Class, Method[])}but allows to pass a
     * list of methods to mock. All the other methods won't be. It means that if
     * these methods are called, their real code will be executed.
     * 
     * @param classToMock
     *            the class to mock
     * @param mockedMethods
     *            Methods to be mocked. If null, all methods will be mocked.
     * @return the mock control
     */
    public static MockClassControl createNiceControl(Class classToMock,
            Method[] mockedMethods) {
        return new MockClassControl(classToMock, mockedMethods,
                NICE_BEHAVIOR_FACTORY);
    }

    /**
     * @deprecated No need to pick a constructor anymore. Constructor arguments
     *             are now ignored. Just use {@link #createControl(Class)}
     */
    public static MockClassControl createControl(Class classToMock,
            Class[] constructorTypes, Object[] constructorArgs) {
        return new MockClassControl(classToMock, UNORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * @deprecated No need to pick a constructor anymore. Constructor arguments
     *             are now ignored. Just use
     *             {@link #createControl(Class, Method[])}
     */
    public static MockClassControl createControl(Class classToMock,
            Class[] constructorTypes, Object[] constructorArgs,
            Method[] mockedMethods) {
        return new MockClassControl(classToMock, mockedMethods,
                UNORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * @deprecated No need to pick a constructor anymore. Constructor arguments
     *             are now ignored. Just use {@link #createStrictControl(Class)}
     */
    public static MockClassControl createStrictControl(Class classToMock,
            Class[] constructorTypes, Object[] constructorArgs) {
        return new MockClassControl(classToMock, ORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * @deprecated No need to pick a constructor anymore. Constructor arguments
     *             are now ignored. Just use
     *             {@link #createStrictControl(Class, Method[])}
     */
    public static MockClassControl createStrictControl(Class classToMock,
            Class[] constructorTypes, Object[] constructorArgs,
            Method[] mockedMethods) {
        return new MockClassControl(classToMock, mockedMethods,
                ORDERED_BEHAVIOR_FACTORY);
    }

    /**
     * @deprecated No need to pick a constructor anymore. Constructor arguments
     *             are now ignored. Just use {@link #createNiceControl(Class)}
     */
    public static MockClassControl createNiceControl(Class classToMock,
            Class[] constructorTypes, Object[] constructorArgs) {
        return new MockClassControl(classToMock, NICE_BEHAVIOR_FACTORY);
    }

    /**
     * @deprecated No need to pick a constructor anymore. Constructor arguments
     *             are now ignored. Just use
     *             {@link #createNiceControl(Class, Method[])}
     */
    public static MockClassControl createNiceControl(Class classToMock,
            Class[] constructorTypes, Object[] constructorArgs,
            Method[] mockedMethods) {
        return new MockClassControl(classToMock, mockedMethods,
                NICE_BEHAVIOR_FACTORY);
    }

    private MockClassControl(Class classToMock, Method[] mockedMethods,
            IBehaviorFactory behaviorFactory) {
        super(classToMock, new ClassProxyFactory(mockedMethods),
                behaviorFactory);
    }

    private MockClassControl(Class classToMock, IBehaviorFactory behaviorFactory) {
        this(classToMock, null, behaviorFactory);
    }
}
